@@ -1,19 +1,12 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 """A module controlling settings for Hypothesis to use in falsification.
 
@@ -21,14 +14,13 @@ Either an explicit settings object can be used or the default object on
 this module can be modified.
 """
 
-from __future__ import absolute_import, division, print_function
-
 import contextlib
 import datetime
 import inspect
-import threading
+import os
 import warnings
 from enum import Enum, IntEnum, unique
+from typing import TYPE_CHECKING, Any, Collection, Dict, List, Optional, TypeVar, Union
 
 import attr
 
@@ -37,25 +29,22 @@ from hypothesis.errors import (
     InvalidArgument,
     InvalidState,
 )
-from hypothesis.internal.compat import integer_types, quiet_raise, string_types
 from hypothesis.internal.reflection import get_pretty_function_description
 from hypothesis.internal.validation import check_type, try_convert
-from hypothesis.utils.conventions import UniqueIdentifier, not_set
+from hypothesis.utils.conventions import not_set
 from hypothesis.utils.dynamicvariables import DynamicVariable
 
-if False:
-    from typing import Any, Dict, List  # noqa
+if TYPE_CHECKING:
+    from hypothesis.database import ExampleDatabase
 
 __all__ = ["settings"]
 
+all_settings: Dict[str, "Setting"] = {}
 
-unlimited = UniqueIdentifier("unlimited")
+T = TypeVar("T")
 
 
-all_settings = {}  # type: Dict[str, Setting]
-
-
-class settingsProperty(object):
+class settingsProperty:
     def __init__(self, name, show_default):
         self.name = name
         self.show_default = show_default
@@ -75,41 +64,34 @@ class settingsProperty(object):
                     result = ExampleDatabase(not_set)
                 return result
             except KeyError:
-                raise AttributeError(self.name)
+                raise AttributeError(self.name) from None
 
     def __set__(self, obj, value):
         obj.__dict__[self.name] = value
 
     def __delete__(self, obj):
-        raise AttributeError("Cannot delete attribute %s" % (self.name,))
+        raise AttributeError(f"Cannot delete attribute {self.name}")
 
     @property
     def __doc__(self):
         description = all_settings[self.name].description
-        deprecation_message = all_settings[self.name].deprecation_message
         default = (
             repr(getattr(settings.default, self.name))
             if self.show_default
             else "(dynamically calculated)"
         )
-        return "\n\n".join(
-            [
-                description,
-                "default value: ``%s``" % (default,),
-                (deprecation_message or "").strip(),
-            ]
-        ).strip()
+        return f"{description}\n\ndefault value: ``{default}``"
 
 
 default_variable = DynamicVariable(None)
 
 
 class settingsMeta(type):
-    def __init__(self, *args, **kwargs):
-        super(settingsMeta, self).__init__(*args, **kwargs)
+    def __init__(cls, *args, **kwargs):
+        super().__init__(*args, **kwargs)
 
     @property
-    def default(self):
+    def default(cls):
         v = default_variable.value
         if v is not None:
             return v
@@ -118,10 +100,10 @@ class settingsMeta(type):
             assert default_variable.value is not None
         return default_variable.value
 
-    def _assign_default_internal(self, value):
+    def _assign_default_internal(cls, value):
         default_variable.value = value
 
-    def __setattr__(self, name, value):
+    def __setattr__(cls, name, value):
         if name == "default":
             raise AttributeError(
                 "Cannot assign to the property settings.default - "
@@ -129,15 +111,15 @@ class settingsMeta(type):
             )
         elif not (isinstance(value, settingsProperty) or name.startswith("_")):
             raise AttributeError(
-                "Cannot assign hypothesis.settings.%s=%r - the settings "
+                f"Cannot assign hypothesis.settings.{name}={value!r} - the settings "
                 "class is immutable.  You can change the global default "
                 "settings with settings.load_profile, or use @settings(...) "
-                "to decorate your test instead." % (name, value)
+                "to decorate your test instead."
             )
-        return type.__setattr__(self, name, value)
+        return super().__setattr__(name, value)
 
 
-class settings(settingsMeta("settings", (object,), {})):  # type: ignore
+class settings(metaclass=settingsMeta):
     """A settings object controls a variety of parameters that are used in
     falsification. These may control both the falsification strategy and the
     details of the data that is generated.
@@ -146,68 +128,76 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
     changes made there will be picked up in newly created settings.
     """
 
-    _WHITELISTED_REAL_PROPERTIES = ["_construction_complete", "storage"]
     __definitions_are_locked = False
-    _profiles = {}  # type: dict
+    _profiles: Dict[str, "settings"] = {}
     __module__ = "hypothesis"
 
     def __getattr__(self, name):
         if name in all_settings:
             return all_settings[name].default
         else:
-            raise AttributeError("settings has no attribute %s" % (name,))
+            raise AttributeError(f"settings has no attribute {name}")
 
-    def __init__(self, parent=None, **kwargs):
-        # type: (settings, **Any) -> None
-        if parent is not None and not isinstance(parent, settings):
-            raise InvalidArgument(
-                "Invalid argument: parent=%r is not a settings instance" % (parent,)
-            )
-        if kwargs.get("derandomize"):
-            if kwargs.get("database") is not None:
+    def __init__(
+        self,
+        parent: Optional["settings"] = None,
+        *,
+        # This looks pretty strange, but there's good reason: we want Mypy to detect
+        # bad calls downstream, but not to freak out about the `= not_set` part even
+        # though it's not semantically valid to pass that as an argument value.
+        # The intended use is "like **kwargs, but more tractable for tooling".
+        max_examples: int = not_set,  # type: ignore
+        derandomize: bool = not_set,  # type: ignore
+        database: Union[None, "ExampleDatabase"] = not_set,  # type: ignore
+        verbosity: "Verbosity" = not_set,  # type: ignore
+        phases: Collection["Phase"] = not_set,  # type: ignore
+        stateful_step_count: int = not_set,  # type: ignore
+        report_multiple_bugs: bool = not_set,  # type: ignore
+        suppress_health_check: Collection["HealthCheck"] = not_set,  # type: ignore
+        deadline: Union[None, int, float, datetime.timedelta] = not_set,  # type: ignore
+        print_blob: bool = not_set,  # type: ignore
+    ) -> None:
+        if parent is not None:
+            check_type(settings, parent, "parent")
+        if derandomize not in (not_set, False):
+            if database not in (not_set, None):
                 raise InvalidArgument(
                     "derandomize=True implies database=None, so passing "
-                    "database=%r too is invalid." % (kwargs["database"],)
+                    f"database={database!r} too is invalid."
                 )
-            kwargs["database"] = None
-        self._construction_complete = False
-        deprecations = []
+            database = None
+
         defaults = parent or settings.default
         if defaults is not None:
             for setting in all_settings.values():
-                if kwargs.get(setting.name, not_set) is not_set:
-                    kwargs[setting.name] = getattr(defaults, setting.name)
+                value = locals()[setting.name]
+                if value is not_set:
+                    object.__setattr__(
+                        self, setting.name, getattr(defaults, setting.name)
+                    )
                 else:
-                    if setting.validator:
-                        kwargs[setting.name] = setting.validator(kwargs[setting.name])
-                    if setting.deprecation_message is not None:
-                        deprecations.append(setting)
-        for name, value in kwargs.items():
-            if name not in all_settings:
-                raise InvalidArgument(
-                    "Invalid argument: %r is not a valid setting" % (name,)
-                )
-            setattr(self, name, value)
-        self.storage = threading.local()
-        self._construction_complete = True
+                    object.__setattr__(self, setting.name, setting.validator(value))
 
-        for d in deprecations:
-            note_deprecation(d.deprecation_message, since=d.deprecated_since)
-
-    def __call__(self, test):
+    def __call__(self, test: T) -> T:
         """Make the settings object (self) an attribute of the test.
 
         The settings are later discovered by looking them up on the test itself.
         """
-        if not callable(test):
+        # Aliasing as Any avoids mypy errors (attr-defined) when accessing and
+        # setting custom attributes on the decorated function or class.
+        _test: Any = test
+
+        # Using the alias here avoids a mypy error (return-value) later when
+        # ``test`` is returned, because this check results in type refinement.
+        if not callable(_test):
             raise InvalidArgument(
                 "settings objects can be called as a decorator with @given, "
-                "but decorated test=%r is not callable." % (test,)
+                f"but decorated test={test!r} is not callable."
             )
         if inspect.isclass(test):
-            from hypothesis.stateful import GenericStateMachine
+            from hypothesis.stateful import RuleBasedStateMachine
 
-            if issubclass(test, GenericStateMachine):
+            if issubclass(_test, RuleBasedStateMachine):
                 attr_name = "_hypothesis_internal_settings_applied"
                 if getattr(test, attr_name, False):
                     raise InvalidArgument(
@@ -216,28 +206,25 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
                         "instead."
                     )
                 setattr(test, attr_name, True)
-                test.TestCase.settings = self
-                return test
+                _test.TestCase.settings = self
+                return test  # type: ignore
             else:
                 raise InvalidArgument(
                     "@settings(...) can only be used as a decorator on "
-                    "functions, or on subclasses of GenericStateMachine."
+                    "functions, or on subclasses of RuleBasedStateMachine."
                 )
-        if hasattr(test, "_hypothesis_internal_settings_applied"):
+        if hasattr(_test, "_hypothesis_internal_settings_applied"):
             # Can't use _hypothesis_internal_use_settings as an indicator that
             # @settings was applied, because @given also assigns that attribute.
+            descr = get_pretty_function_description(test)
             raise InvalidArgument(
-                "%s has already been decorated with a settings object."
-                "\n    Previous:  %r\n    This:  %r"
-                % (
-                    get_pretty_function_description(test),
-                    test._hypothesis_internal_use_settings,
-                    self,
-                )
+                f"{descr} has already been decorated with a settings object.\n"
+                f"    Previous:  {_test._hypothesis_internal_use_settings!r}\n"
+                f"    This:  {self!r}"
             )
 
-        test._hypothesis_internal_use_settings = self
-        test._hypothesis_internal_settings_applied = True
+        _test._hypothesis_internal_use_settings = self
+        _test._hypothesis_internal_settings_applied = True
         return test
 
     @classmethod
@@ -249,8 +236,6 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
         options=None,
         validator=None,
         show_default=True,
-        deprecation_message=None,
-        deprecated_since=None,
     ):
         """Add a new setting.
 
@@ -268,6 +253,13 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
         if options is not None:
             options = tuple(options)
             assert default in options
+
+            def validator(value):
+                if value not in options:
+                    msg = f"Invalid {name}, {value!r}. Valid options: {options!r}"
+                    raise InvalidArgument(msg)
+                return value
+
         else:
             assert validator is not None
 
@@ -275,10 +267,7 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
             name=name,
             description=description.strip(),
             default=default,
-            options=options,
             validator=validator,
-            deprecation_message=deprecation_message,
-            deprecated_since=deprecated_since,
         )
         setattr(settings, name, settingsProperty(name, show_default))
 
@@ -287,46 +276,26 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
         settings.__definitions_are_locked = True
 
     def __setattr__(self, name, value):
-        if name in settings._WHITELISTED_REAL_PROPERTIES:
-            return object.__setattr__(self, name, value)
-        elif name in all_settings:
-            if self._construction_complete:
-                raise AttributeError(
-                    "settings objects are immutable and may not be assigned to"
-                    " after construction."
-                )
-            else:
-                setting = all_settings[name]
-                if setting.options is not None and value not in setting.options:
-                    raise InvalidArgument(
-                        "Invalid %s, %r. Valid options: %r"
-                        % (name, value, setting.options)
-                    )
-                return object.__setattr__(self, name, value)
-        else:
-            raise AttributeError("No such setting %s" % (name,))
+        raise AttributeError("settings objects are immutable")
 
     def __repr__(self):
-        bits = []
-        for name, setting in all_settings.items():
-            value = getattr(self, name)
-            # The only settings that are not shown are those that are
-            # deprecated and left at their default values.
-            if value != setting.default or not setting.deprecation_message:
-                bits.append("%s=%r" % (name, value))
-        return "settings(%s)" % ", ".join(sorted(bits))
+        bits = sorted(f"{name}={getattr(self, name)!r}" for name in all_settings)
+        return "settings({})".format(", ".join(bits))
 
     def show_changed(self):
         bits = []
         for name, setting in all_settings.items():
             value = getattr(self, name)
             if value != setting.default:
-                bits.append("%s=%r" % (name, value))
+                bits.append(f"{name}={value!r}")
         return ", ".join(sorted(bits, key=len))
 
     @staticmethod
-    def register_profile(name, parent=None, **kwargs):
-        # type: (str, settings, **Any) -> None
+    def register_profile(
+        name: str,
+        parent: Optional["settings"] = None,
+        **kwargs: Any,
+    ) -> None:
         """Registers a collection of values to be used as a settings profile.
 
         Settings profiles can be loaded by name - for example, you might
@@ -339,29 +308,27 @@ class settings(settingsMeta("settings", (object,), {})):  # type: ignore
         keyword arguments for each setting that will be set differently to
         parent (or settings.default, if parent is None).
         """
-        check_type(string_types, name, "name")
+        check_type(str, name, "name")
         settings._profiles[name] = settings(parent=parent, **kwargs)
 
     @staticmethod
-    def get_profile(name):
-        # type: (str) -> settings
+    def get_profile(name: str) -> "settings":
         """Return the profile with the given name."""
-        check_type(string_types, name, "name")
+        check_type(str, name, "name")
         try:
             return settings._profiles[name]
         except KeyError:
-            raise InvalidArgument("Profile %r is not registered" % (name,))
+            raise InvalidArgument(f"Profile {name!r} is not registered") from None
 
     @staticmethod
-    def load_profile(name):
-        # type: (str) -> None
+    def load_profile(name: str) -> None:
         """Loads in the settings defined in the profile provided.
 
         If the profile does not exist, InvalidArgument will be raised.
         Any setting not defined in the profile will be the library
         defined default for that setting.
         """
-        check_type(string_types, name, "name")
+        check_type(str, name, "name")
         settings._current_profile = name
         settings._assign_default_internal(settings.get_profile(name))
 
@@ -374,34 +341,19 @@ def local_settings(s):
 
 
 @attr.s()
-class Setting(object):
+class Setting:
     name = attr.ib()
     description = attr.ib()
     default = attr.ib()
-    options = attr.ib()
     validator = attr.ib()
-    deprecation_message = attr.ib()
-    deprecated_since = attr.ib()
-
-
-def _ensure_positive_int(x, name, since, min_value=0):
-    if not isinstance(x, integer_types):
-        note_deprecation(
-            "Passing non-integer %s=%r is deprecated" % (name, x), since=since
-        )
-    x = try_convert(int, x, name)
-    if x < min_value:
-        raise InvalidArgument("%s=%r must be at least %r." % (name, x, min_value))
-    return x
 
 
 def _max_examples_validator(x):
-    x = _ensure_positive_int(x, "max_examples", since="2019-03-06", min_value=0)
-    if x == 0:
-        note_deprecation(
-            "max_examples=%r should be at least one. You can disable example "
-            "generation with the `phases` setting instead." % (x,),
-            since="2019-03-06",
+    check_type(int, x, name="max_examples")
+    if x < 1:
+        raise InvalidArgument(
+            f"max_examples={x!r} should be at least one. You can disable "
+            "example generation with the `phases` setting instead."
         )
     return x
 
@@ -422,47 +374,27 @@ If you are writing one-off tests, running tens of thousands of examples is
 quite reasonable as Hypothesis may miss uncommon bugs with default settings.
 For very complex code, we have observed Hypothesis finding novel bugs after
 *several million* examples while testing :pypi:`SymPy`.
+If you are running more than 100k examples for a test, consider using our
+:ref:`integration for coverage-guided fuzzing <fuzz_one_input>` - it really
+shines when given minutes or hours to run.
 """,
 )
 
-settings._define_setting(
-    "buffer_size",
-    default=not_set,
-    validator=lambda x: _ensure_positive_int(x, "buffer_size", since="2019-03-06"),
-    description="The buffer_size setting has been deprecated and no longer does anything.",
-    deprecation_message="The buffer_size setting can safely be removed with no effect.",
-    deprecated_since="2019-07-03",
-)
-
-
-settings._define_setting(
-    "timeout",
-    default=not_set,
-    description="The timeout setting has been deprecated and no longer does anything.",
-    deprecation_message="The timeout setting can safely be removed with no effect.",
-    deprecated_since="2017-11-02",
-    options=(not_set, unlimited),
-)
-
-
-def _derandomize_validator(x):
-    if not isinstance(x, bool):
-        note_deprecation("derandomize=%r should be a bool." % (x,), since="2019-03-06")
-    return bool(x)
-
 
 settings._define_setting(
     "derandomize",
     default=False,
-    validator=_derandomize_validator,
+    options=(True, False),
     description="""
-If this is True then hypothesis will run in deterministic mode
-where each falsification uses a random number generator that is seeded
-based on the hypothesis to falsify, which will be consistent across
-multiple runs. This has the advantage that it will eliminate any
-randomness from your tests, which may be preferable for some situations.
-It does have the disadvantage of making your tests less likely to
-find novel breakages.
+If True, seed Hypothesis' random number generator using a hash of the test
+function, so that every run will test the same set of examples until you
+update Hypothesis, Python, or the test function.
+
+This allows you to `check for regressions and look for bugs
+<https://blog.nelhage.com/post/two-kinds-of-testing/>`__ using
+:ref:`separate settings profiles <settings_profiles>` - for example running
+quick deterministic tests on every commit, and a longer non-deterministic
+nightly testing run.
 """,
 )
 
@@ -474,9 +406,9 @@ def _validate_database(db):
         return db
     raise InvalidArgument(
         "Arguments to the database setting must be None or an instance of "
-        "ExampleDatabase.  Try passing database=ExampleDatabase(%r), or "
+        f"ExampleDatabase.  Try passing database=ExampleDatabase({db!r}), or "
         "construct and use one of the specific subclasses in "
-        "hypothesis.database" % (db,)
+        "hypothesis.database"
     )
 
 
@@ -485,10 +417,12 @@ settings._define_setting(
     default=not_set,
     show_default=False,
     description="""
-An instance of hypothesis.database.ExampleDatabase that will be
+An instance of :class:`~hypothesis.database.ExampleDatabase` that will be
 used to save examples to and load previous examples from. May be ``None``
-in which case no storage will be used, ``":memory:"`` for an in-memory
-database, or any path for a directory-based example database.
+in which case no storage will be used.
+
+See the :doc:`example database documentation <database>` for a list of built-in
+example database implementations, and how to define custom implementations.
 """,
     validator=_validate_database,
 )
@@ -501,9 +435,10 @@ class Phase(IntEnum):
     generate = 2
     target = 3
     shrink = 4
+    explain = 5
 
     def __repr__(self):
-        return "Phase.%s" % (self.name,)
+        return f"Phase.{self.name}"
 
 
 @unique
@@ -514,17 +449,21 @@ class HealthCheck(Enum):
     """
 
     def __repr__(self):
-        return "%s.%s" % (self.__class__.__name__, self.name)
+        return f"{self.__class__.__name__}.{self.name}"
 
     @classmethod
-    def all(cls):
-        # type: () -> List[HealthCheck]
-        deprecated = [HealthCheck.hung_test]
-        return [x for x in list(HealthCheck) if x not in deprecated]
+    def all(cls) -> List["HealthCheck"]:
+        return list(HealthCheck)
 
     data_too_large = 1
-    """Check for when the typical size of the examples you are generating
-    exceeds the maximum allowed size too often."""
+    """Checks if too many examples are aborted for being too large.
+
+    This is measured by the number of random choices that Hypothesis makes
+    in order to generate something, not the size of the generated object.
+    For example, choosing a 100MB object from a predefined list would take
+    only a few bits, while generating 10KB of JSON from scratch might trigger
+    this health check.
+    """
 
     filter_too_much = 2
     """Check for when the test is filtering out too many examples, either
@@ -539,11 +478,6 @@ class HealthCheck(Enum):
     """Checks if your tests return a non-None value (which will be ignored and
     is unlikely to do what you want)."""
 
-    hung_test = 6
-    """This health check is deprecated and no longer has any effect.
-    You can use the ``max_examples`` and ``deadline`` settings together to cap
-    the total runtime of your tests, rather than the previous fixed limit."""
-
     large_base_example = 7
     """Checks if the natural example to shrink towards is very large."""
 
@@ -551,6 +485,24 @@ class HealthCheck(Enum):
     """Checks if :func:`@given <hypothesis.given>` has been applied to a
     method defined by :class:`python:unittest.TestCase` (i.e. not a test)."""
 
+    function_scoped_fixture = 9
+    """Checks if :func:`@given <hypothesis.given>` has been applied to a test
+    with a pytest function-scoped fixture. Function-scoped fixtures run once
+    for the whole function, not once per example, and this is usually not what
+    you want.
+
+    Because of this limitation, tests that need to set up or reset
+    state for every example need to do so manually within the test itself,
+    typically using an appropriate context manager.
+
+    Suppress this health check only in the rare case that you are using a
+    function-scoped fixture that does not need to be reset between individual
+    examples, but for some reason you cannot use a wider fixture scope
+    (e.g. session scope, module scope, class scope).
+
+    This check requires the :ref:`Hypothesis pytest plugin<pytest-plugin>`,
+    which is enabled by default when running Hypothesis inside pytest."""
+
 
 @unique
 class Verbosity(IntEnum):
@@ -560,7 +512,7 @@ class Verbosity(IntEnum):
     debug = 3
 
     def __repr__(self):
-        return "Verbosity.%s" % (self.name,)
+        return f"Verbosity.{self.name}"
 
 
 settings._define_setting(
@@ -572,19 +524,18 @@ settings._define_setting(
 
 
 def _validate_phases(phases):
-    if phases is None:
-        phases = tuple(Phase)
-        note_deprecation("Use phases=%r, not None." % (phases,), since="2019-08-05")
     phases = tuple(phases)
     for a in phases:
         if not isinstance(a, Phase):
-            raise InvalidArgument("%r is not a valid phase" % (a,))
+            raise InvalidArgument(f"{a!r} is not a valid phase")
     return tuple(p for p in list(Phase) if p in phases)
 
 
 settings._define_setting(
     "phases",
-    default=tuple(Phase),
+    # We leave the `explain` phase disabled by default, for speed and brevity
+    # TODO: consider default-enabling this in CI?
+    default=_validate_phases(set(Phase) - {Phase.explain}),
     description=(
         "Control which phases should be run. "
         "See :ref:`the full documentation for more details <phases>`"
@@ -592,12 +543,18 @@ settings._define_setting(
     validator=_validate_phases,
 )
 
+
+def _validate_stateful_step_count(x):
+    check_type(int, x, name="stateful_step_count")
+    if x < 1:
+        raise InvalidArgument(f"stateful_step_count={x!r} must be at least one.")
+    return x
+
+
 settings._define_setting(
     name="stateful_step_count",
     default=50,
-    validator=lambda x: _ensure_positive_int(
-        x, "stateful_step_count", "2019-03-06", min_value=1
-    ),
+    validator=_validate_stateful_step_count,
     description="""
 Number of steps to run a stateful program for before giving up on it breaking.
 """,
@@ -621,14 +578,8 @@ def validate_health_check_suppressions(suppressions):
     for s in suppressions:
         if not isinstance(s, HealthCheck):
             raise InvalidArgument(
-                "Non-HealthCheck value %r of type %s is invalid in suppress_health_check."
-                % (s, type(s).__name__)
-            )
-        if s is HealthCheck.hung_test:
-            note_deprecation(
-                "HealthCheck.hung_test is deprecated and has no "
-                "effect, as we no longer run this health check.",
-                since="2019-01-24",
+                f"Non-HealthCheck value {s!r} of type {type(s).__name__} "
+                "is invalid in suppress_health_check."
             )
     return suppressions
 
@@ -646,46 +597,42 @@ class duration(datetime.timedelta):
 
     def __repr__(self):
         ms = self.total_seconds() * 1000
-        return "timedelta(milliseconds=%r)" % (int(ms) if ms == int(ms) else ms,)
+        return f"timedelta(milliseconds={int(ms) if ms == int(ms) else ms!r})"
 
 
 def _validate_deadline(x):
-    if isinstance(x, bool):
-        note_deprecation(
-            "The deadline=%r must be a duration in milliseconds, or None to disable."
-            "  Boolean deadlines are treated as ints, and deprecated." % (x,),
-            since="2019-03-06",
-        )
     if x is None:
         return x
-    if isinstance(x, integer_types + (float,)):
+    invalid_deadline_error = InvalidArgument(
+        f"deadline={x!r} (type {type(x).__name__}) must be a timedelta object, "
+        "an integer or float number of milliseconds, or None to disable the "
+        "per-test-case deadline."
+    )
+    if isinstance(x, (int, float)):
+        if isinstance(x, bool):
+            raise invalid_deadline_error
         try:
             x = duration(milliseconds=x)
         except OverflowError:
-            quiet_raise(
-                InvalidArgument(
-                    "deadline=%r is invalid, because it is too large to represent "
-                    "as a timedelta. Use deadline=None to disable deadlines." % (x,)
-                )
-            )
+            raise InvalidArgument(
+                f"deadline={x!r} is invalid, because it is too large to represent "
+                "as a timedelta. Use deadline=None to disable deadlines."
+            ) from None
     if isinstance(x, datetime.timedelta):
         if x <= datetime.timedelta(0):
             raise InvalidArgument(
-                "deadline=%r is invalid, because it is impossible to meet a "
-                "deadline <= 0. Use deadline=None to disable deadlines." % (x,)
+                f"deadline={x!r} is invalid, because it is impossible to meet a "
+                "deadline <= 0. Use deadline=None to disable deadlines."
             )
         return duration(seconds=x.total_seconds())
-    raise InvalidArgument(
-        "deadline=%r (type %s) must be a timedelta object, an integer or float number of milliseconds, "
-        "or None to disable the per-test-case deadline." % (x, type(x).__name__)
-    )
+    raise invalid_deadline_error
 
 
 settings._define_setting(
     "deadline",
     default=duration(milliseconds=200),
     validator=_validate_deadline,
-    description=u"""
+    description="""
 If set, a duration (as timedelta, or integer or float number of milliseconds)
 that each individual example (i.e. each time your test
 function is called, not the whole decorated test) within a test is not
@@ -693,67 +640,54 @@ allowed to exceed. Tests which take longer than that may be converted into
 errors (but will not necessarily be if close to the deadline, to allow some
 variability in test run time).
 
-Set this to None to disable this behaviour entirely.
+Set this to ``None`` to disable this behaviour entirely.
 """,
 )
 
 
-class PrintSettings(Enum):
-    """Flags to determine whether or not to print a detailed example blob to
-    use with :func:`~hypothesis.reproduce_failure` for failing test cases."""
-
-    NEVER = 0
-    """Never print a blob."""
-
-    INFER = 1
-    """This option is deprecated and will be treated as equivalent to
-    ALWAYS."""
-
-    ALWAYS = 2
-    """Always print a blob on failure."""
-
-    def __repr__(self):
-        return "PrintSettings.%s" % (self.name,)
-
-
-def _validate_print_blob(value):
-    if isinstance(value, PrintSettings):
-        replacement = value != PrintSettings.NEVER
-
-        note_deprecation(
-            "Setting print_blob=%r is deprecated and will become an error "
-            "in a future version of Hypothesis. Use print_blob=%r instead."
-            % (value, replacement),
-            since="2018-09-30",
-        )
-        return replacement
-
-    check_type(bool, value, "print_blob")
-
-    return value
+def is_in_ci() -> bool:
+    # GitHub Actions, Travis CI and AppVeyor have "CI"
+    # Azure Pipelines has "TF_BUILD"
+    return "CI" in os.environ or "TF_BUILD" in os.environ
 
 
 settings._define_setting(
     "print_blob",
-    default=False,
+    default=is_in_ci(),
+    show_default=False,
+    options=(True, False),
     description="""
-If set to True, Hypothesis will print code for failing examples that can be used with
+If set to ``True``, Hypothesis will print code for failing examples that can be used with
 :func:`@reproduce_failure <hypothesis.reproduce_failure>` to reproduce the failing example.
+The default is ``True`` if the ``CI`` or ``TF_BUILD`` env vars are set, ``False`` otherwise.
 """,
-    validator=_validate_print_blob,
 )
 
 settings.lock_further_definitions()
 
 
-def note_deprecation(message, since):
-    # type: (str, str) -> None
+def note_deprecation(message: str, *, since: str, has_codemod: bool) -> None:
     if since != "RELEASEDAY":
         date = datetime.datetime.strptime(since, "%Y-%m-%d").date()
         assert datetime.date(2016, 1, 1) <= date
+    if has_codemod:
+        message += (
+            "\n    The `hypothesis codemod` command-line tool can automatically "
+            "refactor your code to fix this warning."
+        )
     warnings.warn(HypothesisDeprecationWarning(message), stacklevel=2)
 
 
 settings.register_profile("default", settings())
 settings.load_profile("default")
 assert settings.default is not None
+
+
+# Check that the kwonly args to settings.__init__ is the same as the set of
+# defined settings - in case we've added or remove something from one but
+# not the other.
+assert set(all_settings) == {
+    p.name
+    for p in inspect.signature(settings.__init__).parameters.values()
+    if p.kind == inspect.Parameter.KEYWORD_ONLY
+}
