@@ -1,52 +1,53 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
-# https://github.com/HypothesisWorks/hypothesis-python
+# https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2018 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
-#
-# END HEADER
-
-from __future__ import division, print_function, absolute_import
+# obtain one at https://mozilla.org/MPL/2.0/.
 
+from collections import OrderedDict, abc
 from copy import copy
-from collections import Iterable, OrderedDict
+from datetime import datetime, timedelta
+from typing import Any, List, Optional, Sequence, Set, Union
 
 import attr
 import numpy as np
-
 import pandas
-import hypothesis.strategies as st
-import hypothesis.extra.numpy as npst
-import hypothesis.internal.conjecture.utils as cu
-from hypothesis.errors import InvalidArgument
+
+from hypothesis import strategies as st
+from hypothesis._settings import note_deprecation
 from hypothesis.control import reject
-from hypothesis.internal.compat import hrange
+from hypothesis.errors import InvalidArgument
+from hypothesis.extra import numpy as npst
+from hypothesis.internal.conjecture import utils as cu
 from hypothesis.internal.coverage import check, check_function
-from hypothesis.internal.validation import check_type, try_convert, \
-    check_strategy, check_valid_size, check_valid_interval
+from hypothesis.internal.reflection import get_pretty_function_description
+from hypothesis.internal.validation import (
+    check_type,
+    check_valid_interval,
+    check_valid_size,
+    try_convert,
+)
+from hypothesis.strategies._internal.strategies import Ex, check_strategy
+from hypothesis.strategies._internal.utils import cacheable, defines_strategy
 
 try:
     from pandas.api.types import is_categorical_dtype
-except ImportError:  # pragma: no cover
+except ImportError:
+
     def is_categorical_dtype(dt):
         if isinstance(dt, np.dtype):
             return False
-        return dt == 'category'
+        return dt == "category"
 
 
 def dtype_for_elements_strategy(s):
     return st.shared(
         s.map(lambda x: pandas.Series([x]).dtype),
-        key=('hypothesis.extra.pandas.dtype_for_elements_strategy', s),
+        key=("hypothesis.extra.pandas.dtype_for_elements_strategy", s),
     )
 
 
@@ -60,47 +61,53 @@ def infer_dtype_if_necessary(dtype, values, elements, draw):
 def elements_and_dtype(elements, dtype, source=None):
 
     if source is None:
-        prefix = ''
+        prefix = ""
     else:
-        prefix = '%s.' % (source,)
+        prefix = f"{source}."
 
     if elements is not None:
-        check_strategy(elements, '%selements' % (prefix,))
+        check_strategy(elements, f"{prefix}elements")
     else:
-        with check('dtype is not None'):
+        with check("dtype is not None"):
             if dtype is None:
-                raise InvalidArgument((
-                    'At least one of %(prefix)selements or %(prefix)sdtype '
-                    'must be provided.') % {'prefix': prefix})
+                raise InvalidArgument(
+                    f"At least one of {prefix}elements or {prefix}dtype must be provided."
+                )
 
-    with check('is_categorical_dtype'):
+    with check("is_categorical_dtype"):
         if is_categorical_dtype(dtype):
             raise InvalidArgument(
-                '%sdtype is categorical, which is currently unsupported' % (
-                    prefix,
-                ))
+                f"{prefix}dtype is categorical, which is currently unsupported"
+            )
 
-    dtype = try_convert(np.dtype, dtype, 'dtype')
+    if isinstance(dtype, type) and np.dtype(dtype).kind == "O" and dtype is not object:
+        note_deprecation(
+            f"Passed dtype={dtype!r} is not a valid Pandas dtype.  We'll treat it as "
+            "dtype=object for now, but this will be an error in a future version.",
+            since="2021-12-31",
+            has_codemod=False,
+        )
+
+    dtype = try_convert(np.dtype, dtype, "dtype")
 
     if elements is None:
         elements = npst.from_dtype(dtype)
     elif dtype is not None:
+
         def convert_element(value):
-            name = 'draw(%selements)' % (prefix,)
+            name = f"draw({prefix}elements)"
             try:
                 return np.array([value], dtype=dtype)[0]
             except TypeError:
                 raise InvalidArgument(
-                    'Cannot convert %s=%r of type %s to dtype %s' % (
-                        name, value, type(value).__name__, dtype.str
-                    )
-                )
+                    "Cannot convert %s=%r of type %s to dtype %s"
+                    % (name, value, type(value).__name__, dtype.str)
+                ) from None
             except ValueError:
                 raise InvalidArgument(
-                    'Cannot convert %s=%r to type %s' % (
-                        name, value, dtype.str,
-                    )
-                )
+                    f"Cannot convert {name}={value!r} to type {dtype.str}"
+                ) from None
+
         elements = elements.map(convert_element)
     assert elements is not None
 
@@ -109,7 +116,7 @@ def elements_and_dtype(elements, dtype, source=None):
 
 class ValueIndexStrategy(st.SearchStrategy):
     def __init__(self, elements, dtype, min_size, max_size, unique):
-        super(ValueIndexStrategy, self).__init__()
+        super().__init__()
         self.elements = elements
         self.dtype = dtype
         self.min_size = min_size
@@ -121,8 +128,10 @@ class ValueIndexStrategy(st.SearchStrategy):
         seen = set()
 
         iterator = cu.many(
-            data, min_size=self.min_size, max_size=self.max_size,
-            average_size=(self.min_size + self.max_size) / 2
+            data,
+            min_size=self.min_size,
+            max_size=self.max_size,
+            average_size=(self.min_size + self.max_size) / 2,
         )
 
         while iterator.more():
@@ -136,8 +145,7 @@ class ValueIndexStrategy(st.SearchStrategy):
             result.append(elt)
 
         dtype = infer_dtype_if_necessary(
-            dtype=self.dtype, values=result, elements=self.elements,
-            draw=data.draw
+            dtype=self.dtype, values=result, elements=self.elements, draw=data.draw
         )
         return pandas.Index(result, dtype=dtype, tupleize_cols=False)
 
@@ -145,9 +153,12 @@ class ValueIndexStrategy(st.SearchStrategy):
 DEFAULT_MAX_SIZE = 10
 
 
-@st.cacheable
-@st.defines_strategy
-def range_indexes(min_size=0, max_size=None):
+@cacheable
+@defines_strategy()
+def range_indexes(
+    min_size: int = 0,
+    max_size: Optional[int] = None,
+) -> st.SearchStrategy[pandas.RangeIndex]:
     """Provides a strategy which generates an :class:`~pandas.Index` whose
     values are 0, 1, ..., n for some n.
 
@@ -157,19 +168,24 @@ def range_indexes(min_size=0, max_size=None):
     * max_size is the largest number of elements the index can have. If None
       it will default to some suitable value based on min_size.
     """
-    check_valid_size(min_size, 'min_size')
-    check_valid_size(max_size, 'max_size')
+    check_valid_size(min_size, "min_size")
+    check_valid_size(max_size, "max_size")
     if max_size is None:
-        max_size = min([min_size + DEFAULT_MAX_SIZE, 2 ** 63 - 1])
-    check_valid_interval(min_size, max_size, 'min_size', 'max_size')
+        max_size = min([min_size + DEFAULT_MAX_SIZE, 2**63 - 1])
+    check_valid_interval(min_size, max_size, "min_size", "max_size")
     return st.integers(min_size, max_size).map(pandas.RangeIndex)
 
 
-@st.cacheable
-@st.defines_strategy
+@cacheable
+@defines_strategy()
 def indexes(
-    elements=None, dtype=None, min_size=0, max_size=None, unique=True,
-):
+    *,
+    elements: Optional[st.SearchStrategy[Ex]] = None,
+    dtype: Any = None,
+    min_size: int = 0,
+    max_size: Optional[int] = None,
+    unique: bool = True,
+) -> st.SearchStrategy[pandas.Index]:
     """Provides a strategy for producing a :class:`pandas.Index`.
 
     Arguments:
@@ -189,21 +205,27 @@ def indexes(
     * unique specifies whether all of the elements in the resulting index
       should be distinct.
     """
-    check_valid_size(min_size, 'min_size')
-    check_valid_size(max_size, 'max_size')
-    check_valid_interval(min_size, max_size, 'min_size', 'max_size')
-    check_type(bool, unique, 'unique')
+    check_valid_size(min_size, "min_size")
+    check_valid_size(max_size, "max_size")
+    check_valid_interval(min_size, max_size, "min_size", "max_size")
+    check_type(bool, unique, "unique")
 
     elements, dtype = elements_and_dtype(elements, dtype)
 
     if max_size is None:
         max_size = min_size + DEFAULT_MAX_SIZE
-    return ValueIndexStrategy(
-        elements, dtype, min_size, max_size, unique)
+    return ValueIndexStrategy(elements, dtype, min_size, max_size, unique)
 
 
-@st.defines_strategy
-def series(elements=None, dtype=None, index=None, fill=None, unique=False):
+@defines_strategy()
+def series(
+    *,
+    elements: Optional[st.SearchStrategy[Ex]] = None,
+    dtype: Any = None,
+    index: Optional[st.SearchStrategy[Union[Sequence, pandas.Index]]] = None,
+    fill: Optional[st.SearchStrategy[Ex]] = None,
+    unique: bool = False,
+) -> st.SearchStrategy[pandas.Series]:
     """Provides a strategy for producing a :class:`pandas.Series`.
 
     Arguments:
@@ -240,7 +262,7 @@ def series(elements=None, dtype=None, index=None, fill=None, unique=False):
     if index is None:
         index = range_indexes()
     else:
-        check_strategy(index)
+        check_strategy(index, "index")
 
     elements, dtype = elements_and_dtype(elements, dtype)
     index_strategy = index
@@ -251,30 +273,43 @@ def series(elements=None, dtype=None, index=None, fill=None, unique=False):
 
         if len(index) > 0:
             if dtype is not None:
-                result_data = draw(npst.arrays(
-                    dtype=dtype, elements=elements, shape=len(index),
-                    fill=fill, unique=unique,
-                ))
+                result_data = draw(
+                    npst.arrays(
+                        dtype=dtype,
+                        elements=elements,
+                        shape=len(index),
+                        fill=fill,
+                        unique=unique,
+                    )
+                )
             else:
-                result_data = list(draw(npst.arrays(
-                    dtype=object, elements=elements, shape=len(index),
-                    fill=fill, unique=unique,
-                )))
+                result_data = list(
+                    draw(
+                        npst.arrays(
+                            dtype=object,
+                            elements=elements,
+                            shape=len(index),
+                            fill=fill,
+                            unique=unique,
+                        )
+                    )
+                )
 
-            return pandas.Series(
-                result_data, index=index, dtype=dtype
-            )
+            return pandas.Series(result_data, index=index, dtype=dtype)
         else:
             return pandas.Series(
-                (), index=index,
-                dtype=dtype if dtype is not None else draw(
-                    dtype_for_elements_strategy(elements)))
+                (),
+                index=index,
+                dtype=dtype
+                if dtype is not None
+                else draw(dtype_for_elements_strategy(elements)),
+            )
 
     return result()
 
 
 @attr.s(slots=True)
-class column(object):
+class column:
     """Data object for describing a column in a DataFrame.
 
     Arguments:
@@ -293,14 +328,19 @@ class column(object):
 
     name = attr.ib(default=None)
     elements = attr.ib(default=None)
-    dtype = attr.ib(default=None)
+    dtype = attr.ib(default=None, repr=get_pretty_function_description)
     fill = attr.ib(default=None)
     unique = attr.ib(default=False)
 
 
 def columns(
-    names_or_number, dtype=None, elements=None, fill=None, unique=False
-):
+    names_or_number: Union[int, Sequence[str]],
+    *,
+    dtype: Any = None,
+    elements: Optional[st.SearchStrategy[Ex]] = None,
+    fill: Optional[st.SearchStrategy[Ex]] = None,
+    unique: bool = False,
+) -> List[column]:
     """A convenience function for producing a list of :class:`column` objects
     of the same general shape.
 
@@ -310,21 +350,23 @@ def columns(
     be created. All other arguments are passed through verbatim to
     create the columns.
     """
-    try:
+    if isinstance(names_or_number, (int, float)):
+        names: List[Union[int, str, None]] = [None] * names_or_number
+    else:
         names = list(names_or_number)
-    except TypeError:
-        names = [None] * names_or_number
     return [
-        column(
-            name=n, dtype=dtype, elements=elements, fill=fill, unique=unique
-        ) for n in names
+        column(name=n, dtype=dtype, elements=elements, fill=fill, unique=unique)
+        for n in names
     ]
 
 
-@st.defines_strategy
+@defines_strategy()
 def data_frames(
-    columns=None, rows=None, index=None
-):
+    columns: Optional[Sequence[column]] = None,
+    *,
+    rows: Optional[st.SearchStrategy[Union[dict, Sequence[Any]]]] = None,
+    index: Optional[st.SearchStrategy[Ex]] = None,
+) -> st.SearchStrategy[pandas.DataFrame]:
     """Provides a strategy for producing a :class:`pandas.DataFrame`.
 
     Arguments:
@@ -430,16 +472,15 @@ def data_frames(
     if index is None:
         index = range_indexes()
     else:
-        check_strategy(index)
+        check_strategy(index, "index")
 
     index_strategy = index
 
     if columns is None:
         if rows is None:
-            raise InvalidArgument(
-                'At least one of rows and columns must be provided'
-            )
+            raise InvalidArgument("At least one of rows and columns must be provided")
         else:
+
             @st.composite
             def rows_only(draw):
                 index = draw(index_strategy)
@@ -447,34 +488,32 @@ def data_frames(
                 @check_function
                 def row():
                     result = draw(rows)
-                    check_type(Iterable, result, 'draw(row)')
+                    check_type(abc.Iterable, result, "draw(row)")
                     return result
 
                 if len(index) > 0:
-                    return pandas.DataFrame(
-                        [row() for _ in index],
-                        index=index
-                    )
+                    return pandas.DataFrame([row() for _ in index], index=index)
                 else:
                     # If we haven't drawn any rows we need to draw one row and
                     # then discard it so that we get a consistent shape for the
                     # DataFrame.
                     base = pandas.DataFrame([row()])
                     return base.drop(0)
+
             return rows_only()
 
     assert columns is not None
-    columns = try_convert(tuple, columns, 'columns')
+    cols = try_convert(tuple, columns, "columns")
 
     rewritten_columns = []
-    column_names = set()
+    column_names: Set[str] = set()
 
-    for i, c in enumerate(columns):
-        check_type(column, c, 'columns[%d]' % (i,))
+    for i, c in enumerate(cols):
+        check_type(column, c, f"columns[{i}]")
 
         c = copy(c)
         if c.name is None:
-            label = 'columns[%d]' % (i,)
+            label = f"columns[{i}]"
             c.name = i
         else:
             label = c.name
@@ -482,34 +521,31 @@ def data_frames(
                 hash(c.name)
             except TypeError:
                 raise InvalidArgument(
-                    'Column names must be hashable, but columns[%d].name was '
-                    '%r of type %s, which cannot be hashed.' % (
-                        i, c.name, type(c.name).__name__,))
+                    "Column names must be hashable, but columns[%d].name was "
+                    "%r of type %s, which cannot be hashed."
+                    % (i, c.name, type(c.name).__name__)
+                ) from None
 
         if c.name in column_names:
-            raise InvalidArgument(
-                'duplicate definition of column name %r' % (c.name,))
+            raise InvalidArgument(f"duplicate definition of column name {c.name!r}")
 
         column_names.add(c.name)
 
-        c.elements, c.dtype = elements_and_dtype(
-            c.elements, c.dtype, label
-        )
+        c.elements, c.dtype = elements_and_dtype(c.elements, c.dtype, label)
 
         if c.dtype is None and rows is not None:
             raise InvalidArgument(
-                'Must specify a dtype for all columns when combining rows with'
-                ' columns.'
+                "Must specify a dtype for all columns when combining rows with columns."
             )
 
         c.fill = npst.fill_for(
-            fill=c.fill, elements=c.elements, unique=c.unique,
-            name=label
+            fill=c.fill, elements=c.elements, unique=c.unique, name=label
         )
 
         rewritten_columns.append(c)
 
     if rows is None:
+
         @st.composite
         def just_draw_columns(draw):
             index = draw(index_strategy)
@@ -530,19 +566,16 @@ def data_frames(
             # row wise, so that the values of each row are next to each other.
             # This makes life easier for the shrinker when deleting blocks of
             # data.
-            columns_without_fill = [
-                c for c in rewritten_columns if c.fill.is_empty]
+            columns_without_fill = [c for c in rewritten_columns if c.fill.is_empty]
 
             if columns_without_fill:
                 for c in columns_without_fill:
                     data[c.name] = pandas.Series(
-                        np.zeros(shape=len(index), dtype=c.dtype),
-                        index=index,
+                        np.zeros(shape=len(index), dtype=c.dtype), index=index
                     )
-                seen = {
-                    c.name: set() for c in columns_without_fill if c.unique}
+                seen = {c.name: set() for c in columns_without_fill if c.unique}
 
-                for i in hrange(len(index)):
+                for i in range(len(index)):
                     for c in columns_without_fill:
                         if c.unique:
                             for _ in range(5):
@@ -554,39 +587,67 @@ def data_frames(
                                 reject()
                         else:
                             value = draw(c.elements)
-                        data[c.name][i] = value
+                        try:
+                            data[c.name][i] = value
+                        except ValueError as err:  # pragma: no cover
+                            # This just works in Pandas 1.4 and later, but gives
+                            # a confusing error on previous versions.
+                            if c.dtype is None and not isinstance(
+                                value, (float, int, str, bool, datetime, timedelta)
+                            ):
+                                raise ValueError(
+                                    f"Failed to add value={value!r} to column "
+                                    f"{c.name} with dtype=None.  Maybe passing "
+                                    "dtype=object would help?"
+                                ) from err
+                            # Unclear how this could happen, but users find a way...
+                            raise
 
             for c in rewritten_columns:
                 if not c.fill.is_empty:
-                    data[c.name] = draw(series(
-                        index=local_index_strategy, dtype=c.dtype,
-                        elements=c.elements, fill=c.fill, unique=c.unique))
+                    data[c.name] = draw(
+                        series(
+                            index=local_index_strategy,
+                            dtype=c.dtype,
+                            elements=c.elements,
+                            fill=c.fill,
+                            unique=c.unique,
+                        )
+                    )
 
             return pandas.DataFrame(data, index=index)
+
         return just_draw_columns()
     else:
+
         @st.composite
         def assign_rows(draw):
             index = draw(index_strategy)
 
-            result = pandas.DataFrame(OrderedDict(
-                (c.name, pandas.Series(
-                    np.zeros(dtype=c.dtype, shape=len(index)), dtype=c.dtype))
-                for c in rewritten_columns
-            ), index=index)
+            result = pandas.DataFrame(
+                OrderedDict(
+                    (
+                        c.name,
+                        pandas.Series(
+                            np.zeros(dtype=c.dtype, shape=len(index)), dtype=c.dtype
+                        ),
+                    )
+                    for c in rewritten_columns
+                ),
+                index=index,
+            )
 
             fills = {}
 
             any_unique = any(c.unique for c in rewritten_columns)
 
             if any_unique:
-                all_seen = [
-                    set() if c.unique else None for c in rewritten_columns]
+                all_seen = [set() if c.unique else None for c in rewritten_columns]
                 while all_seen[-1] is None:
                     all_seen.pop()
 
-            for row_index in hrange(len(index)):
-                for _ in hrange(5):
+            for row_index in range(len(index)):
+                for _ in range(5):
                     original_row = draw(rows)
                     row = original_row
                     if isinstance(row, dict):
@@ -598,16 +659,19 @@ def data_frames(
                                 try:
                                     as_list[i] = fills[i]
                                 except KeyError:
+                                    if c.fill.is_empty:
+                                        raise InvalidArgument(
+                                            f"Empty fill strategy in {c!r} cannot "
+                                            f"complete row {original_row!r}"
+                                        ) from None
                                     fills[i] = draw(c.fill)
                                     as_list[i] = fills[i]
                         for k in row:
                             if k not in column_names:
-                                raise InvalidArgument((
-                                    'Row %r contains column %r not in '
-                                    'columns %r)' % (
-                                        row, k, [
-                                            c.name for c in rewritten_columns
-                                        ])))
+                                raise InvalidArgument(
+                                    "Row %r contains column %r not in columns %r)"
+                                    % (row, k, [c.name for c in rewritten_columns])
+                                )
                         row = as_list
                     if any_unique:
                         has_duplicate = False
@@ -620,19 +684,25 @@ def data_frames(
                             seen.add(value)
                         if has_duplicate:
                             continue
-                    row = list(try_convert(tuple, row, 'draw(rows)'))
+                    row = list(try_convert(tuple, row, "draw(rows)"))
 
                     if len(row) > len(rewritten_columns):
-                        raise InvalidArgument((
-                            'Row %r contains too many entries. Has %d but '
-                            'expected at most %d') % (
-                                original_row, len(row), len(rewritten_columns)
-                        ))
+                        raise InvalidArgument(
+                            f"Row {original_row!r} contains too many entries. Has "
+                            f"{len(row)} but expected at most {len(rewritten_columns)}"
+                        )
                     while len(row) < len(rewritten_columns):
-                        row.append(draw(rewritten_columns[len(row)].fill))
+                        c = rewritten_columns[len(row)]
+                        if c.fill.is_empty:
+                            raise InvalidArgument(
+                                f"Empty fill strategy in {c!r} cannot "
+                                f"complete row {original_row!r}"
+                            )
+                        row.append(draw(c.fill))
                     result.iloc[row_index] = row
                     break
                 else:
                     reject()
             return result
+
         return assign_rows()
