@@ -1,72 +1,56 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 """This file can approximately be considered the collection of hypothesis going
 to really unreasonable lengths to produce pretty output."""
 
-
-from __future__ import absolute_import, division, print_function
-
 import ast
 import hashlib
 import inspect
+import os
 import re
-import tokenize
+import sys
 import types
-import uuid
 from functools import wraps
+from tokenize import detect_encoding
 from types import ModuleType
+from typing import TYPE_CHECKING, Callable
+from unittest.mock import _patch as PatchType
 
-from hypothesis.configuration import storage_directory
-from hypothesis.internal.compat import (
-    ARG_NAME_ATTRIBUTE,
-    getfullargspec,
-    hrange,
-    isidentifier,
-    qualname,
-    str_to_bytes,
-    to_str,
-    to_unicode,
-    update_code_location,
-)
+from hypothesis.internal.compat import is_typed_named_tuple, update_code_location
+from hypothesis.utils.conventions import not_set
 from hypothesis.vendor.pretty import pretty
 
+if TYPE_CHECKING:
+    from hypothesis.strategies._internal.strategies import T
 
-def fully_qualified_name(f):
-    """Returns a unique identifier for f pointing to the module it was defined
-    on, and an containing functions."""
-    if f.__module__ is not None:
-        return f.__module__ + "." + qualname(f)
-    else:
-        return qualname(f)
+READTHEDOCS = os.environ.get("READTHEDOCS", None) == "True"
 
 
 def is_mock(obj):
-    """Determine if the given argument is a mock type.
+    """Determine if the given argument is a mock type."""
 
-    We want to be able to detect these when dealing with various test
-    args. As they are sneaky and can look like almost anything else,
-    we'll check this by looking for random attributes.  This is more
-    robust than looking for types.
-    """
-    for _ in range(10):
-        if not hasattr(obj, str(uuid.uuid4())):
-            return False
-    return True
+    # We want to be able to detect these when dealing with various test
+    # args. As they are sneaky and can look like almost anything else,
+    # we'll check this by looking for an attribute with a name that it's really
+    # unlikely to implement accidentally, and that anyone who implements it
+    # deliberately should know what they're doing. This is more robust than
+    # looking for types.
+    return hasattr(obj, "hypothesis_internal_is_this_a_mock_check")
+
+
+def getfullargspec_except_self(target):
+    spec = inspect.getfullargspec(target)
+    if inspect.ismethod(target):
+        del spec.args[0]
+    return spec
 
 
 def function_digest(function):
@@ -76,22 +60,21 @@ def function_digest(function):
 
     No guarantee of uniqueness though it usually will be.
     """
-    hasher = hashlib.md5()
+    hasher = hashlib.sha384()
     try:
-        hasher.update(to_unicode(inspect.getsource(function)).encode("utf-8"))
-    # Different errors on different versions of python. What fun.
-    except (OSError, IOError, TypeError):
+        hasher.update(inspect.getsource(function).encode())
+    except (OSError, TypeError):
         pass
     try:
-        hasher.update(str_to_bytes(function.__name__))
+        hasher.update(function.__name__.encode())
     except AttributeError:
         pass
     try:
-        hasher.update(function.__module__.__name__.encode("utf-8"))
+        hasher.update(function.__module__.__name__.encode())
     except AttributeError:
         pass
     try:
-        hasher.update(str_to_bytes(repr(getfullargspec(function))))
+        hasher.update(repr(getfullargspec_except_self(function)).encode())
     except TypeError:
         pass
     try:
@@ -101,18 +84,48 @@ def function_digest(function):
     return hasher.digest()
 
 
-def is_typed_named_tuple(cls):
-    """Return True if cls is probably a subtype of `typing.NamedTuple`.
+def get_signature(target):
+    # Special case for use of `@unittest.mock.patch` decorator, mimicking the
+    # behaviour of getfullargspec instead of reporting unusable arguments.
+    patches = getattr(target, "patchings", None)
+    if isinstance(patches, list) and all(isinstance(p, PatchType) for p in patches):
+        P = inspect.Parameter
+        return inspect.Signature(
+            [P("args", P.VAR_POSITIONAL), P("keywargs", P.VAR_KEYWORD)]
+        )
 
-    Unfortunately types created with `class T(NamedTuple):` actually
-    subclass `tuple` directly rather than NamedTuple.  This is annoying,
-    and means we just have to hope that nobody defines a different tuple
-    subclass with similar attributes.
-    """
-    return (
-        issubclass(cls, tuple)
-        and hasattr(cls, "_fields")
-        and hasattr(cls, "_field_types")
+    if isinstance(getattr(target, "__signature__", None), inspect.Signature):
+        # This special case covers unusual codegen like Pydantic models
+        sig = target.__signature__
+        # And *this* much more complicated block ignores the `self` argument
+        # if that's been (incorrectly) included in the custom signature.
+        if sig.parameters and (inspect.isclass(target) or inspect.ismethod(target)):
+            selfy = next(iter(sig.parameters.values()))
+            if (
+                selfy.name == "self"
+                and selfy.default is inspect.Parameter.empty
+                and selfy.kind.name.startswith("POSITIONAL_")
+            ):
+                return sig.replace(
+                    parameters=[v for k, v in sig.parameters.items() if k != "self"]
+                )
+        return sig
+    if sys.version_info[:2] <= (3, 8) and inspect.isclass(target):
+        # Workaround for subclasses of typing.Generic on Python <= 3.8
+        from hypothesis.strategies._internal.types import is_generic_type
+
+        if is_generic_type(target):
+            sig = inspect.signature(target.__init__)
+            return sig.replace(
+                parameters=[v for k, v in sig.parameters.items() if k != "self"]
+            )
+    return inspect.signature(target)
+
+
+def arg_is_required(param):
+    return param.default is inspect.Parameter.empty and param.kind in (
+        inspect.Parameter.POSITIONAL_OR_KEYWORD,
+        inspect.Parameter.KEYWORD_ONLY,
     )
 
 
@@ -129,34 +142,24 @@ def required_args(target, args=(), kwargs=()):
     if inspect.isclass(target) and is_typed_named_tuple(target):
         provided = set(kwargs) | set(target._fields[: len(args)])
         return set(target._fields) - provided
-    # Then we try to do the right thing with getfullargspec
+    # Then we try to do the right thing with inspect.signature
     try:
-        spec = getfullargspec(
-            getattr(target, "__init__", target) if inspect.isclass(target) else target
-        )
-    except TypeError:  # pragma: no cover
-        return None
-    # self appears in the argspec of __init__ and bound methods, but it's an
-    # error to explicitly supply it - so we might skip the first argument.
-    skip_self = int(inspect.isclass(target) or inspect.ismethod(target))
-    # Start with the args that were not supplied and all kwonly arguments,
-    # then remove all positional arguments with default values, and finally
-    # remove kwonly defaults and any supplied keyword arguments
-    return (
-        set(spec.args[skip_self + len(args) :] + spec.kwonlyargs)
-        - set(spec.args[len(spec.args) - len(spec.defaults or ()) :])
-        - set(spec.kwonlydefaults or ())
-        - set(kwargs)
-    )
+        sig = get_signature(target)
+    except (ValueError, TypeError):
+        return set()
+    return {
+        name
+        for name, param in list(sig.parameters.items())[len(args) :]
+        if arg_is_required(param) and name not in kwargs
+    }
 
 
 def convert_keyword_arguments(function, args, kwargs):
     """Returns a pair of a tuple and a dictionary which would be equivalent
-    passed as positional and keyword args to the function. Unless function has.
-
-    **kwargs the dictionary will always be empty.
+    passed as positional and keyword args to the function. Unless function has
+    kwonlyargs or **kwargs the dictionary will always be empty.
     """
-    argspec = getfullargspec(function)
+    argspec = getfullargspec_except_self(function)
     new_args = []
     kwargs = dict(kwargs)
 
@@ -170,7 +173,7 @@ def convert_keyword_arguments(function, args, kwargs):
 
     n = max(len(args), len(argspec.args))
 
-    for i in hrange(n):
+    for i in range(n):
         if i < len(args):
             new_args.append(args[i])
         else:
@@ -180,9 +183,9 @@ def convert_keyword_arguments(function, args, kwargs):
             elif arg_name in defaults:
                 new_args.append(defaults[arg_name])
             else:
-                raise TypeError("No value provided for argument %r" % (arg_name))
+                raise TypeError(f"No value provided for argument {arg_name!r}")
 
-    if kwargs and not argspec.varkw:
+    if kwargs and not (argspec.varkw or argspec.kwonlyargs):
         if len(kwargs) > 1:
             raise TypeError(
                 "%s() got unexpected keyword arguments %s"
@@ -191,8 +194,7 @@ def convert_keyword_arguments(function, args, kwargs):
         else:
             bad_kwarg = next(iter(kwargs))
             raise TypeError(
-                "%s() got an unexpected keyword argument %r"
-                % (function.__name__, bad_kwarg)
+                f"{function.__name__}() got an unexpected keyword argument {bad_kwarg!r}"
             )
     return tuple(new_args), kwargs
 
@@ -203,59 +205,70 @@ def convert_positional_arguments(function, args, kwargs):
 
     new_args will only be non-empty if function has a variadic argument.
     """
-    argspec = getfullargspec(function)
+    argspec = getfullargspec_except_self(function)
     new_kwargs = dict(argspec.kwonlydefaults or {})
     new_kwargs.update(kwargs)
     if not argspec.varkw:
         for k in new_kwargs.keys():
             if k not in argspec.args and k not in argspec.kwonlyargs:
                 raise TypeError(
-                    "%s() got an unexpected keyword argument %r"
-                    % (function.__name__, k)
+                    f"{function.__name__}() got an unexpected keyword argument {k!r}"
                 )
     if len(args) < len(argspec.args):
-        for i in hrange(len(args), len(argspec.args) - len(argspec.defaults or ())):
+        for i in range(len(args), len(argspec.args) - len(argspec.defaults or ())):
             if argspec.args[i] not in kwargs:
-                raise TypeError(
-                    "No value provided for argument %s" % (argspec.args[i],)
-                )
+                raise TypeError(f"No value provided for argument {argspec.args[i]}")
     for kw in argspec.kwonlyargs:
         if kw not in new_kwargs:
-            raise TypeError("No value provided for argument %s" % kw)
+            raise TypeError(f"No value provided for argument {kw}")
 
     if len(args) > len(argspec.args) and not argspec.varargs:
         raise TypeError(
-            "%s() takes at most %d positional arguments (%d given)"
-            % (function.__name__, len(argspec.args), len(args))
+            f"{function.__name__}() takes at most {len(argspec.args)} "
+            f"positional arguments ({len(args)} given)"
         )
 
     for arg, name in zip(args, argspec.args):
         if name in new_kwargs:
             raise TypeError(
-                "%s() got multiple values for keyword argument %r"
-                % (function.__name__, name)
+                f"{function.__name__}() got multiple values for keyword argument {name!r}"
             )
         else:
             new_kwargs[name] = arg
+
     return (tuple(args[len(argspec.args) :]), new_kwargs)
 
 
-def extract_all_lambdas(tree):
+def ast_arguments_matches_signature(args, sig):
+    assert isinstance(args, ast.arguments)
+    assert isinstance(sig, inspect.Signature)
+    expected = []
+    for node in getattr(args, "posonlyargs", ()):  # New in Python 3.8
+        expected.append((node.arg, inspect.Parameter.POSITIONAL_ONLY))
+    for node in args.args:
+        expected.append((node.arg, inspect.Parameter.POSITIONAL_OR_KEYWORD))
+    if args.vararg is not None:
+        expected.append((args.vararg.arg, inspect.Parameter.VAR_POSITIONAL))
+    for node in args.kwonlyargs:
+        expected.append((node.arg, inspect.Parameter.KEYWORD_ONLY))
+    if args.kwarg is not None:
+        expected.append((args.kwarg.arg, inspect.Parameter.VAR_KEYWORD))
+    return expected == [(p.name, p.kind) for p in sig.parameters.values()]
+
+
+def extract_all_lambdas(tree, matching_signature):
     lambdas = []
 
     class Visitor(ast.NodeVisitor):
         def visit_Lambda(self, node):
-            lambdas.append(node)
+            if ast_arguments_matches_signature(node.args, matching_signature):
+                lambdas.append(node)
 
     Visitor().visit(tree)
 
     return lambdas
 
 
-def args_for_lambda_ast(l):
-    return [getattr(n, ARG_NAME_ATTRIBUTE) for n in l.args.args]
-
-
 LINE_CONTINUATION = re.compile(r"\\\n")
 WHITESPACE = re.compile(r"\s+")
 PROBABLY_A_COMMENT = re.compile("""#[^'"]*$""")
@@ -270,36 +283,15 @@ def extract_lambda_source(f):
     This is not a good function and I am sorry for it. Forgive me my
     sins, oh lord
     """
-    argspec = getfullargspec(f)
-    arg_strings = []
-    # In Python 2 you can have destructuring arguments to functions. This
-    # results in an argspec with non-string values. I'm not very interested in
-    # handling these properly, but it's important to not crash on them.
-    bad_lambda = False
-    for a in argspec.args:
-        if isinstance(a, (tuple, list)):  # pragma: no cover
-            arg_strings.append("(%s)" % (", ".join(a),))
-            bad_lambda = True
-        else:
-            assert isinstance(a, str)
-            arg_strings.append(a)
-    if argspec.varargs:
-        arg_strings.append("*" + argspec.varargs)
-    elif argspec.kwonlyargs:
-        arg_strings.append("*")
-    for a in argspec.kwonlyargs or []:
-        default = (argspec.kwonlydefaults or {}).get(a)
-        if default:
-            arg_strings.append("{}={}".format(a, default))
-        else:
-            arg_strings.append(a)
-
-    if_confused = "lambda %s: <unknown>" % (", ".join(arg_strings),)
-    if bad_lambda:  # pragma: no cover
-        return if_confused
+    sig = inspect.signature(f)
+    assert sig.return_annotation is inspect.Parameter.empty
+    if sig.parameters:
+        if_confused = f"lambda {str(sig)[1:-1]}: <unknown>"
+    else:
+        if_confused = "lambda: <unknown>"
     try:
         source = inspect.getsource(f)
-    except IOError:
+    except OSError:
         return if_confused
 
     source = LINE_CONTINUATION.sub(" ", source)
@@ -312,7 +304,7 @@ def extract_lambda_source(f):
     try:
         tree = ast.parse(source)
     except SyntaxError:
-        for i in hrange(len(source) - 1, len("lambda"), -1):
+        for i in range(len(source) - 1, len("lambda"), -1):
             prefix = source[:i]
             if "lambda" not in prefix:
                 break
@@ -322,28 +314,27 @@ def extract_lambda_source(f):
                 break
             except SyntaxError:
                 continue
-    if tree is None:
-        if source.startswith("@"):
-            # This will always eventually find a valid expression because
-            # the decorator must be a valid Python function call, so will
-            # eventually be syntactically valid and break out of the loop. Thus
-            # this loop can never terminate normally, so a no branch pragma is
-            # appropriate.
-            for i in hrange(len(source) + 1):  # pragma: no branch
-                p = source[1:i]
-                if "lambda" in p:
-                    try:
-                        tree = ast.parse(p)
-                        source = p
-                        break
-                    except SyntaxError:
-                        pass
+    if tree is None and source.startswith("@"):
+        # This will always eventually find a valid expression because
+        # the decorator must be a valid Python function call, so will
+        # eventually be syntactically valid and break out of the loop.
+        # Thus, this loop can never terminate normally.
+        for i in range(len(source) + 1):
+            p = source[1:i]
+            if "lambda" in p:
+                try:
+                    tree = ast.parse(p)
+                    source = p
+                    break
+                except SyntaxError:
+                    pass
+        else:
+            raise NotImplementedError("expected to be unreachable")
 
     if tree is None:
         return if_confused
 
-    all_lambdas = extract_all_lambdas(tree)
-    aligned_lambdas = [l for l in all_lambdas if args_for_lambda_ast(l) == argspec.args]
+    aligned_lambdas = extract_all_lambdas(tree, matching_signature=sig)
     if len(aligned_lambdas) != 1:
         return if_confused
     lambda_ast = aligned_lambdas[0]
@@ -366,21 +357,14 @@ def extract_lambda_source(f):
     # an OSError.  Or if `f` is a built-in function, in which case we get a
     # TypeError.  In both cases, fall back to splitting the Unicode string.
     # It's not perfect, but it's the best we can do.
-    #
-    # Note 2: You can only detect the encoding with `tokenize.detect_encoding`
-    # in Python 3.2 or later.  But that's okay, because the only version that
-    # affects for us is Python 2.7, and 2.7 doesn't support non-ASCII identifiers:
-    # https://www.python.org/dev/peps/pep-3131/. In this case we'll get an
-    # AttributeError.
-    #
     try:
         with open(inspect.getsourcefile(f), "rb") as src_f:
-            encoding, _ = tokenize.detect_encoding(src_f.readline)
+            encoding, _ = detect_encoding(src_f.readline)
 
         source_bytes = source.encode(encoding)
         source_bytes = source_bytes[lambda_ast.col_offset :].strip()
         source = source_bytes.decode(encoding)
-    except (AttributeError, OSError, TypeError):
+    except (OSError, TypeError):
         source = source[lambda_ast.col_offset :].strip()
 
     # This ValueError can be thrown in Python 3 if:
@@ -398,7 +382,7 @@ def extract_lambda_source(f):
     except ValueError:
         return if_confused
 
-    for i in hrange(len(source), len("lambda"), -1):  # pragma: no branch
+    for i in range(len(source), len("lambda"), -1):  # pragma: no branch
         try:
             parsed = ast.parse(source[:i])
             assert len(parsed.body) == 1
@@ -424,12 +408,16 @@ def get_pretty_function_description(f):
         return repr(f)
     name = f.__name__
     if name == "<lambda>":
-        result = extract_lambda_source(f)
-        return result
-    elif isinstance(f, types.MethodType):
+        return extract_lambda_source(f)
+    elif isinstance(f, (types.MethodType, types.BuiltinMethodType)):
         self = f.__self__
-        if not (self is None or inspect.isclass(self)):
-            return "%r.%s" % (self, name)
+        # Some objects, like `builtins.abs` are of BuiltinMethodType but have
+        # their module as __self__.  This might include c-extensions generally?
+        if not (self is None or inspect.isclass(self) or inspect.ismodule(self)):
+            return f"{self!r}.{name}"
+    elif isinstance(name, str) and getattr(dict, name, object()) is f:
+        # special case for keys/values views in from_type() / ghostwriter output
+        return f"dict.{name}"
     return name
 
 
@@ -439,43 +427,32 @@ def nicerepr(v):
     elif isinstance(v, type):
         return v.__name__
     else:
-        return to_str(pretty(v))
+        # With TypeVar T, show List[T] instead of TypeError on List[~T]
+        return re.sub(r"(\[)~([A-Z][a-z]*\])", r"\g<1>\g<2>", pretty(v))
 
 
 def arg_string(f, args, kwargs, reorder=True):
     if reorder:
         args, kwargs = convert_positional_arguments(f, args, kwargs)
 
-    argspec = getfullargspec(f)
+    bits = [nicerepr(x) for x in args]
 
-    bits = []
-
-    for a in argspec.args:
-        if a in kwargs:
-            bits.append("%s=%s" % (a, nicerepr(kwargs.pop(a))))
+    for p in get_signature(f).parameters.values():
+        if p.name in kwargs and not p.kind.name.startswith("VAR_"):
+            bits.append(f"{p.name}={nicerepr(kwargs.pop(p.name))}")
     if kwargs:
         for a in sorted(kwargs):
-            bits.append("%s=%s" % (a, nicerepr(kwargs[a])))
+            bits.append(f"{a}={nicerepr(kwargs[a])}")
 
-    return ", ".join([nicerepr(x) for x in args] + bits)
-
-
-def unbind_method(f):
-    """Take something that might be a method or a function and return the
-    underlying function."""
-    return getattr(f, "im_func", getattr(f, "__func__", f))
+    return ", ".join(bits)
 
 
 def check_valid_identifier(identifier):
-    if not isidentifier(identifier):
-        raise ValueError("%r is not a valid python identifier" % (identifier,))
+    if not identifier.isidentifier():
+        raise ValueError(f"{identifier!r} is not a valid python identifier")
 
 
-def eval_directory():
-    return storage_directory("eval_source")
-
-
-eval_cache = {}  # type: dict
+eval_cache: dict = {}
 
 
 def source_exec_as_module(source):
@@ -484,27 +461,22 @@ def source_exec_as_module(source):
     except KeyError:
         pass
 
-    result = ModuleType(
-        "hypothesis_temporary_module_%s"
-        % (hashlib.sha1(str_to_bytes(source)).hexdigest(),)
-    )
+    hexdigest = hashlib.sha384(source.encode()).hexdigest()
+    result = ModuleType("hypothesis_temporary_module_" + hexdigest)
     assert isinstance(source, str)
     exec(source, result.__dict__)
     eval_cache[source] = result
     return result
 
 
-COPY_ARGSPEC_SCRIPT = (
-    """
+COPY_ARGSPEC_SCRIPT = """
 from hypothesis.utils.conventions import not_set
 
-def accept(%(funcname)s):
-    def %(name)s(%(argspec)s):
-        return %(funcname)s(%(invocation)s)
-    return %(name)s
-""".strip()
-    + "\n"
-)
+def accept({funcname}):
+    def {name}({argspec}):
+        return {funcname}({invocation})
+    return {name}
+""".lstrip()
 
 
 def define_function_signature(name, docstring, argspec):
@@ -523,7 +495,7 @@ def define_function_signature(name, docstring, argspec):
         for a in argspec.args[:-n_defaults]:
             parts.append(a)
         for a in argspec.args[-n_defaults:]:
-            parts.append("%s=not_set" % (a,))
+            parts.append(f"{a}=not_set")
     else:
         parts = list(argspec.args)
     used_names = list(argspec.args) + list(argspec.kwonlyargs)
@@ -533,12 +505,12 @@ def define_function_signature(name, docstring, argspec):
         check_valid_identifier(a)
 
     def accept(f):
-        fargspec = getfullargspec(f)
+        fargspec = getfullargspec_except_self(f)
         must_pass_as_kwargs = []
         invocation_parts = []
         for a in argspec.args:
             if a not in fargspec.args and not fargspec.varargs:
-                must_pass_as_kwargs.append(a)
+                must_pass_as_kwargs.append(a)  # pragma: no cover
             else:
                 invocation_parts.append(a)
         if argspec.varargs:
@@ -548,12 +520,12 @@ def define_function_signature(name, docstring, argspec):
         elif argspec.kwonlyargs:
             parts.append("*")
         for k in must_pass_as_kwargs:
-            invocation_parts.append("%(k)s=%(k)s" % {"k": k})
+            invocation_parts.append(f"{k}={k}")  # pragma: no cover
 
         for k in argspec.kwonlyargs:
-            invocation_parts.append("%(k)s=%(k)s" % {"k": k})
+            invocation_parts.append(f"{k}={k}")
             if k in (argspec.kwonlydefaults or []):
-                parts.append("%(k)s=not_set" % {"k": k})
+                parts.append(f"{k}=not_set")
             else:
                 parts.append(k)
         if argspec.varkw:
@@ -561,25 +533,19 @@ def define_function_signature(name, docstring, argspec):
             parts.append("**" + argspec.varkw)
             invocation_parts.append("**" + argspec.varkw)
 
-        candidate_names = ["f"] + [
-            "f_%d" % (i,) for i in hrange(1, len(used_names) + 2)
-        ]
+        candidate_names = ["f"] + [f"f_{i}" for i in range(1, len(used_names) + 2)]
 
         for funcname in candidate_names:  # pragma: no branch
             if funcname not in used_names:
                 break
 
-        base_accept = source_exec_as_module(
-            COPY_ARGSPEC_SCRIPT
-            % {
-                "name": name,
-                "funcname": funcname,
-                "argspec": ", ".join(parts),
-                "invocation": ", ".join(invocation_parts),
-            }
-        ).accept
-
-        result = base_accept(f)
+        source = COPY_ARGSPEC_SCRIPT.format(
+            name=name,
+            funcname=funcname,
+            argspec=", ".join(parts),
+            invocation=", ".join(invocation_parts),
+        )
+        result = source_exec_as_module(source).accept(f)
         result.__doc__ = docstring
         result.__defaults__ = argspec.defaults
         if argspec.kwonlydefaults:
@@ -591,6 +557,111 @@ def define_function_signature(name, docstring, argspec):
     return accept
 
 
+COPY_SIGNATURE_SCRIPT = """
+from hypothesis.utils.conventions import not_set
+
+def accept({funcname}):
+    def {name}{signature}:
+        return {funcname}({invocation})
+    return {name}
+""".lstrip()
+
+
+def get_varargs(sig, kind=inspect.Parameter.VAR_POSITIONAL):
+    for p in sig.parameters.values():
+        if p.kind is kind:
+            return p
+    return None
+
+
+def define_function_signature_from_signature(name, docstring, signature):
+    """A decorator which sets the name, argspec and docstring of the function
+    passed into it."""
+    # TODO: we will (eventually...) replace the last few uses of getfullargspec
+    # with this version, and then delete the one above.  For now though, this
+    # works for @proxies() and @given() is under stricter constraints anyway.
+
+    check_valid_identifier(name)
+    for a in signature.parameters:
+        check_valid_identifier(a)
+
+    used_names = list(signature.parameters) + [name]
+
+    newsig = signature.replace(
+        parameters=[
+            p if p.default is signature.empty else p.replace(default=not_set)
+            for p in (
+                p.replace(annotation=signature.empty)
+                for p in signature.parameters.values()
+            )
+        ],
+        return_annotation=signature.empty,
+    )
+
+    pos_args = [
+        p
+        for p in signature.parameters.values()
+        if p.kind.name.startswith("POSITIONAL_")
+    ]
+
+    def accept(f):
+        fsig = inspect.signature(f)
+        must_pass_as_kwargs = []
+        invocation_parts = []
+        for p in pos_args:
+            if p.name not in fsig.parameters and get_varargs(fsig) is None:
+                must_pass_as_kwargs.append(p.name)
+            else:
+                invocation_parts.append(p.name)
+        if get_varargs(signature) is not None:
+            invocation_parts.append("*" + get_varargs(signature).name)
+        for k in must_pass_as_kwargs:
+            invocation_parts.append(f"{k}={k}")
+        for p in signature.parameters.values():
+            if p.kind is p.KEYWORD_ONLY:
+                invocation_parts.append(f"{p.name}={p.name}")
+        varkw = get_varargs(signature, kind=inspect.Parameter.VAR_KEYWORD)
+        if varkw:
+            invocation_parts.append("**" + varkw.name)
+
+        candidate_names = ["f"] + [f"f_{i}" for i in range(1, len(used_names) + 2)]
+
+        for funcname in candidate_names:  # pragma: no branch
+            if funcname not in used_names:
+                break
+
+        source = COPY_SIGNATURE_SCRIPT.format(
+            name=name,
+            funcname=funcname,
+            signature=str(newsig),
+            invocation=", ".join(invocation_parts),
+        )
+        result = source_exec_as_module(source).accept(f)
+        result.__doc__ = docstring
+        result.__defaults__ = tuple(
+            p.default
+            for p in signature.parameters.values()
+            if p.default is not signature.empty and "POSITIONAL" in p.kind.name
+        )
+        kwdefaults = {
+            p.name: p.default
+            for p in signature.parameters.values()
+            if p.default is not signature.empty and p.kind is p.KEYWORD_ONLY
+        }
+        if kwdefaults:
+            result.__kwdefaults__ = kwdefaults
+        annotations = {
+            p.name: p.annotation
+            for p in signature.parameters.values()
+            if p.annotation is not signature.empty
+        }
+        if annotations:
+            result.__annotations__ = annotations
+        return result
+
+    return accept
+
+
 def impersonate(target):
     """Decorator to update the attributes of a function so that to external
     introspectors it will appear to be the target function.
@@ -612,14 +683,19 @@ def impersonate(target):
     return accept
 
 
-def proxies(target):
+def proxies(target: "T") -> Callable[[Callable], "T"]:
+    replace_sig = define_function_signature_from_signature(
+        target.__name__.replace("<lambda>", "_lambda_"),  # type: ignore
+        target.__doc__,
+        get_signature(target),
+    )
+
     def accept(proxy):
-        return impersonate(target)(
-            wraps(target)(
-                define_function_signature(
-                    target.__name__, target.__doc__, getfullargspec(target)
-                )(proxy)
-            )
-        )
+        return impersonate(target)(wraps(target)(replace_sig(proxy)))
 
     return accept
+
+
+def is_identity_function(f):
+    # TODO: pattern-match the AST to handle `def ...` identity functions too
+    return bool(re.fullmatch(r"lambda (\w+): \1", get_pretty_function_description(f)))
